
public class Tree
{
  private TreeNode root;
  
  // constrcuts a new empty tree
  public Tree()
  { root = null; }  

  // returns the node containing the smallest value in the tree
  // rooted at t when t is not null; otherwise, returns null;
  // the method runs in time O(h), where h is the height
  // of the tree rooted at t
  private TreeNode minNode(TreeNode t)
  {
    if (t != null)
    {
      while (t.getLeft() != null)
        t = t.getLeft();
    }
    return t;
  }

  // returns the node containing the largest value in the tree
  // rooted at t when t is not null; otherwise, returns null;
  // the method runs in time O(h), where h is the height
  // of the tree rooted at t
  private TreeNode maxNode(TreeNode t)
  {
    if (t != null)
    {
      while (t.getRight() != null)
        t = t.getRight();
    }
    return t;
  }

  /*** Part (a) ***/

  // returns true if all nodes in this tree have correct parent links;
  // otherwise returns false;
  // each node should be the parent of its left and right children;
  // the root node's parent should be null
  private boolean verifyParentLinks()
  {
    return verifyParentLinksHelper(root, null);
  }

  private boolean verifyParentLinksHelper(TreeNode t, TreeNode parent)
  {
    return (t == null ||
                (t.getParent() == parent && 
                     verifyParentLinksHelper(t.getLeft(), t) &&
                     verifyParentLinksHelper(t.getRight(), t)));
  }

  /*** Part (b) ***/

  // returns the node which is the successor of t in this tree;
  // returns null if t contains the maximum value in this tree;
  // the method runs in time O(h), where h is the height
  // of the tree rooted at t
  // precondition:  t is a node in this tree;
  //                all nodes in this tree have correct parent links
  private TreeNode successor(TreeNode t)
  {
    // if (t == null)
    //   return null;

    TreeNode rightChild = t.getRight();

    if (rightChild != null)
      return minNode(rightChild);

    TreeNode parent = t.getParent();

    while (parent != null && parent.getRight() == t)

    // Or:
    //  while (parent != null && ((Comparable)parent.getValue()).compareTo(t.getValue()) < 0)

    {
      t = parent;
      parent = t.getParent();
    }


    return parent;
  }

  public void traverse(TreeNode t)
  {
    if (t != null)
    {
      traverse(t.getLeft());

      System.out.print(t.getValue() + " ");
      TreeNode s = successor(t);
      if (s != null)
        System.out.print("==> " + s.getValue() + " ");

      TreeNode temp = t.getParent();
      t.setParent(null);
      System.out.println(" links OK " + verifyParentLinks());
      t.setParent(temp);

      traverse(t.getRight());
    }
  }

  public static void main(String[] args)
  {
    /*
                    G
                /      \
              D          L
           /     \      / \
         A         F   I   M
          \       /   / \ 
           B     E   H   J
            \             \
             C              K
    */

    Tree tree = new Tree();

    TreeNode c = new TreeNode("C", null, null);
    TreeNode k = new TreeNode("K", null, null);
    TreeNode b = new TreeNode("B", null, c);
    TreeNode e = new TreeNode("E", null, null);
    TreeNode h = new TreeNode("H", null, null);
    TreeNode j = new TreeNode("J", null, k);
    TreeNode a = new TreeNode("A", null, b);
    TreeNode f = new TreeNode("F", e, null);
    TreeNode i = new TreeNode("I", h, j);
    TreeNode m = new TreeNode("M", null, null);
    TreeNode d = new TreeNode("D", a, f);
    TreeNode l = new TreeNode("L", i, m);
    TreeNode g = new TreeNode("G", d, l);

    c.setParent(b);
    k.setParent(j);
    b.setParent(a);
    e.setParent(f);
    h.setParent(i);
    j.setParent(i);
    a.setParent(d);
    f.setParent(d);
    i.setParent(l);
    m.setParent(l);
    d.setParent(g);
    l.setParent(g);
    g.setParent(null);

    tree.root = g;
    tree.traverse(tree.root);
  }
}
