import java.util.ArrayList;
import java.util.*;

public class ArrayQueue implements Queue // but NOT Iterator
{
	private ArrayList queue;
	private int modifyCount;
	
    public ArrayQueue()
    {
        queue = new ArrayList();
        modifyCount = 0;
    } 
	
    public void enqueue(Object x)
    {
    	modifyCount++;
        queue.add(x);
    }
    
    public Object dequeue()
    {
    	modifyCount++;
        return queue.remove(0);
    }
    
    public Object peekFront()
    {
        return queue.get(0);
    }
    
    public boolean isEmpty()
    {
        return queue.size() == 0;
    }
    
    public String toString()
    {
    	return queue.toString();
    }
    
    public java.util.Iterator iterator()
	{
		return new Elements();
	}
	
	
	// inner class to implement iterator over list
	private class Elements implements java.util.Iterator
	{
		private int curr = 0;
		private boolean nextCalled = false;
		private int knownMods = modifyCount;
		
		public boolean hasNext()
		{
			verifyMods();
			
			return curr < queue.size();
		}
		
		public Object next()
		{
			verifyMods();
			
			Object result;
			
			if (hasNext())
			{
				nextCalled = true;
				result = queue.get(curr);
				curr++;
				return result;
			}
			else
				throw new java.util.NoSuchElementException();
		}
		
		public void remove()
		{
			//throw new UnsupportedOperationException();
			
			verifyMods();
			
			if (!nextCalled)
				throw new IllegalStateException("Iterator called without call to next().");
			
			queue.remove(curr-1);
			curr--;
			nextCalled = false;
		}
		
		private void verifyMods()
		{
			if (knownMods != modifyCount)
				throw new ConcurrentModificationException("Data structure modified outside iterator.");
		}
    }
    
    
    public static void main(String[] args)
    {
		ArrayQueue q = new ArrayQueue();
		q.enqueue("mark");
		q.enqueue("zuul");
		q.enqueue("minion");
		q.enqueue("judy");
		q.enqueue("judy4");
		q.enqueue("judyqw");
		q.enqueue("judy6");
		q.enqueue("suite judy blue eyes");
		q.enqueue("judy");
		Iterator it = q.iterator();
		Iterator it2 = q.iterator();
		
		while (it.hasNext())
		{
			System.out.println(it.next());
			it2.next();
			it2.remove();
		}
    }
} 
