import java.util.*;
public class StackandQueue
{
    /**
     * Returns the minimal element in a stack of Strings leaving the
     * stack unchanged when the function returns.
     * @param stack the stack of Strings
     * @return the lexicographically smallest String in stack
     */
    public static String findMin(Stack stack)
    {
		String min = (String) stack.pop();
		Stack copy = new ArrayStack();
		copy.push(min);
		while (! stack.isEmpty())
		{
			String str = (String) stack.pop();
			if (str.compareTo(min) < 0)
				min = str;
			copy.push(str);
		}
		//stack = copy;
		while (!copy.isEmpty())
		{
			stack.push(copy.pop());
		}
		
		return min;
	}

	/**
	 * Returns an ArrayList containing elements in s with the
	 * first element of the ArrayList the first (bottom) element
	 * of s.  Leaves s unchanged when the function returns.
	 * @param s the stack
	 * @return an ArrayList with the elements of s (top of s is at end of ArrayList)
	 */
	public static ArrayList stackToList(Stack s)
	{
		ArrayList list;
		
		list = new ArrayList();
		
		while (!s.isEmpty())
			list.add(0,s.pop());  // O(N^2) - the horror!
		
		for (int i = 0; i < list.size(); i++)
			s.push(list.get(i));
		/*
		while (!s.isEmpty())
			list.add(s.pop());  // O(N) - much better!
		
		for (int i = 0; i < list.size()/2; i++)  // O(?)
		{
			Object temp = list.get(i);
			list.set(i, list.get(list.size() - 1 - i));
			list.set(list.size()-1-i, temp);
		}
		
		for (int i = 0; i < list.size(); i++)
			s.push(list.get(i));
		*/
		/*
		Object o = s.pop();
		if (s.isEmpty())
		{
			list = new ArrayList();
			s.push(o);
			list.add(o);
			return list;
		}
		
		list = stackToList(s);  // also O(N), and recursive to boot!
		s.push(o);
		list.add(o);
		*/
		return list;
		
		
	}
    

    /**
     * Reverses the order of the elements in q, runs in O(n)
     * time for an n-element Queue.
     * @param q the queue
     */
    public static void reverse(Queue q)
    {
		Stack s = new ArrayStack();
		while (!q.isEmpty())
		{
			s.push(q.dequeue());
		}
		
		while (!s.isEmpty())
		{
			q.enqueue(s.pop());
		}
		/*
		ArrayList list = new ArrayList();
		
		while (!q.isEmpty())
			list.add(q.dequeue());
			
		for (int k = list.size()-1; k >= 0; k--)
			q.enqueue(list.get(k));
		*/
    }

    public static void main(String args[])
    {
		String[] list = {"zebra", "monkey", "tiger", "dog", "elephant"};
		if (args.length > 1)
			list = args;

		Stack s = new ArrayStack();
		for(int k=0; k < list.length; k++)
			s.push(list[k]);

		String min = findMin(s);
		System.out.println("min = " + min);
		System.out.println("here's the stack:");
		if (s.isEmpty())
			System.out.println("Empty!");
		while (! s.isEmpty())
	    	System.out.println(s.pop());

	    Queue q = new ListQueue();
	    for(int k=0; k < list.length; k++)
			q.enqueue(list[k]);
		System.out.println("\nOriginal queue: " + q);
		reverse(q);
		System.out.println("Reversed queue: " + q);


		Stack s2 = new ArrayStack();
		s2.push(new Integer(8));
		s2.push(new Integer(5));
		s2.push(new Integer(2));
		s2.push(new Integer(1));
			
		System.out.println("Stack = " + s2);
		
		ArrayList a = stackToList(s2);
		System.out.println("After stackToList, \nStack = " + s2);
		System.out.println("ArrayList = " + a);
		
    }
}
